# refer to: 
# https://stackoverflow.com/questions/64095396/detecting-collisions-between-polygons-and-rectangles-in-pygame/64106246#64106246
# and
# https://stackoverflow.com/questions/59553156/pygame-detecting-collision-of-a-rotating-rectangle/59553589#59553589
import numpy as np
import matplotlib.pyplot as plt

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches

import sys
import os
from pathlib import Path
FILE_THIS = Path(__file__).resolve()
PARENT = FILE_THIS.parent
GPARENT = FILE_THIS.parents[1]
GGPARENT = FILE_THIS.parents[2]
sys.path.append(str(PARENT))
sys.path.append(str(GPARENT))
sys.path.append(str(GGPARENT))

import utils.utils as utils

class Rectangle:
    def __init__(self, center, angle, dims):
        self.W_o = dims[0]
        self.L_o = dims[1]
        self.center = center
        self.angle = angle
        # position of the corners in the object frame
        self.corners_o = np.array( [[-self.W_o/ 2.0, -self.L_o/ 2.0], 
                                    [ self.W_o/ 2.0, -self.L_o/ 2.0], 
                                    [ self.W_o/ 2.0,  self.L_o/ 2.0], 
                                    [-self.W_o/ 2.0,  self.L_o/ 2.0],
                                    [-self.W_o/ 2.0, -self.L_o/ 2.0]]).T   
        
        # get_rect_points(self):
        """
        returns four corners of the rectangle.
        bottom left is the first conrner, from there it goes
        counter clockwise.
        """
        # position of the corner in the world frame
        self.corners_wr = np.dot(utils.rot_M(self.angle), self.corners_o)      
        self.corners_w = self.corners_wr + np.array([[self.center[0]],[self.center[1]]])
        self.corners_w = self.corners_w.T

    def update_rectangle(self, center, angle):
        self.center = center
        self.angle = angle
        # position of the corner in the world frame
        self.corners_wr = np.dot(utils.rot_M(self.angle), self.corners_o)      
        self.corners_w = self.corners_wr + np.array([[self.center[0]],[self.center[1]]])
        self.corners_w = self.corners_w.T

    def plot_rect(self, ax, color='r'):
        # rect_rob_pos_rec = mpatches.Rectangle((self.corners_w[0][0], self.corners_w[0][1]), self.W_o, self.L_o, np.rad2deg(self.angle), fc=(0, 1, 1, 0.1), ec=(0, 0, 1, 0.8))
        # rec_patch = ax.add_artist(rect_rob_pos_rec)
        ax.plot(self.corners_w[:,0], self.corners_w[:,1], color)
        mid_point = 0.5 * (self.corners_w[0] + self.corners_w[2])
        plt.scatter(mid_point[0], mid_point[1], marker='*')
        ax.grid(visible=True, ls='-.')   

def collideLineLine(P0, P1, Q0, Q1):  
    d = (P1[0]-P0[0]) * (Q1[1]-Q0[1]) + (P1[1]-P0[1]) * (Q0[0]-Q1[0]) 
    if d == 0:
        return False
    t = ((Q0[0]-P0[0]) * (Q1[1]-Q0[1]) + (Q0[1]-P0[1]) * (Q0[0]-Q1[0])) / d
    u = ((Q0[0]-P0[0]) * (P1[1]-P0[1]) + (Q0[1]-P0[1]) * (P0[0]-P1[0])) / d
    return 0 <= t <= 1 and 0 <= u <= 1

def colideRectLine(rect, p1, p2):
    return (collideLineLine(p1, p2, rect.corners_w[0], rect.corners_w[1]) or
            collideLineLine(p1, p2, rect.corners_w[1], rect.corners_w[2]) or
            collideLineLine(p1, p2, rect.corners_w[2], rect.corners_w[3]) or
            collideLineLine(p1, p2, rect.corners_w[3], rect.corners_w[0]))

def collideRectPolygon(rect1, rect2):
    for i in range(np.shape(rect2.corners_w)[0]-1):
        if colideRectLine(rect1, rect2.corners_w[i], rect2.corners_w[i+1]):
            return True
    return False


if __name__ == '__main__':
    # example 1
    ax = plt.subplot(111)
    st = Rectangle([0.0, 0.2], np.pi/3, [0.6, 0.41])
    gripper = Rectangle([-0.34, 0.0], np.pi/2, [0.21,0.16])

    st.plot_rect(ax)
    gripper.plot_rect(ax,'y')
    plt.xlim([-2, 2])
    plt.ylim([-2, 2])
    ax.set_aspect('equal')
    plt.show()
    print(f"gripper and rectangle intersect: {collideRectPolygon(st, gripper)}")