syms r_r W_o x_r y_r theta_r fai d
syms d_x_r d_y_r d_theta_r d_fai d_d
syms dd_x_r dd_y_r dd_theta_r dd_fai dd_d
syms m_r m_o

% position of the point p in robot frame
R_p_p = [(r_r + 0.5*W_o) * cos(fai); (r_r + 0.5*W_o) * sin(fai)];

% position of the object in robot frame
R_p_o = [(r_r + 0.5*W_o) * cos(fai) + d/sin(fai); (r_r + 0.5*W_o) * sin(fai) + d/cos(fai)];

% velocity of the object in robot frame
d_R_p_o = [-(r_r + 0.5*W_o) * sin(fai)*d_fai + d_d/sin(fai) - d/(sin(fai)^2)*cos(fai)*d_fai;...
           (r_r + 0.5*W_o) * cos(fai)*d_fai  + d_d/cos(fai) + d/(cos(fai)^2)*sin(fai)*d_fai];

% acceleration of the object in robot frame
dd_R_p_o = [-(r_r + 0.5*W_o) * cos(fai)*d_fai*d_fai - (r_r + 0.5*W_o) * sin(fai)*dd_fai +...
            dd_d/sin(fai) - d_d/(sin(fai)^2)*cos(fai)*d_fai...
            - d_d/(sin(fai)^2)*cos(fai)*d_fai + 2*d/(sin(fai)^3)*cos(fai)*cos(fai)*d_fai*d_fai ...
            + d/(sin(fai)^2)*sin(fai)*d_fai*d_fai - d/(sin(fai)^2)*cos(fai)*dd_fai;...
           - (r_r + 0.5*W_o) * sin(fai)*d_fai*d_fai  + (r_r + 0.5*W_o) * cos(fai)*dd_fai + ...
           dd_d/cos(fai) + d_d/(cos(fai)^2)*sin(fai)*d_fai...
           + d_d/(cos(fai)^2)*sin(fai)*d_fai + 2*d/(cos(fai)^3)*sin(fai)*cos(fai)*d_fai*d_fai...
           + d/(cos(fai)^2)*cos(fai)*d_fai*d_fai + d/(cos(fai)^2)*sin(fai)*dd_fai];

% position of the object in world frame
W_p_o = [x_r; y_r] + R_theta(theta_r)*R_p_o;

% velocity of the object in world frame
d_W_p_o = [d_x_r; d_y_r] + d_R_theta(theta_r, d_theta_r) * R_p_o + R_theta(theta_r) * d_R_p_o;

% acceleration of the object in world frame
dd_W_p_o = [dd_x_r; dd_y_r] + dd_R_theta(theta_r, d_theta_r, dd_theta_r) * R_p_o + 2*d_R_theta(theta_r, d_theta_r) * d_R_p_o...
            + R_theta(theta_r)*dd_R_p_o;

% acceleration of the object in robot direction
dd_W_p_o_robdir = R_theta(-theta_r)*dd_W_p_o;

% first equation
% Summing the forces in the free-body diagram of the robot in x_r direction
% m_r*acc_x_r + F_p*cos(pi-fai) = 0 ---(1)
% Summing the forces in the free-body diagram of the object in x_r direction 
% F_p*cos(pi-fai) + (R_W2R*dd_W_p_o).x = 0 ---(2)
% eq1 = (1) - (2) = 0
eq1 = m_r*dd_x_r - m_o*dd_W_p_o_robdir(1);


diff(eq1, dd_x_r)   % 1
diff(eq1, dd_y_r)   % 2
diff(eq1, dd_theta_r)   % 3
diff(eq1, dd_fai)   % 4
diff(eq1, dd_d)     % 5

%%
% velocity of the object on the contact point in world frame
W_v_co = d_W_p_o + R_theta(theta_r + fai) * [-d_theta_r*d; -d_theta_r*0.5*W_o];

% velocity of the object on the contact point in object direction
% O_v_c1 = R_theta(- theta_r - fai) * W_v_co;
O_v_co = R_theta(- theta_r - fai) * d_W_p_o + [-d_theta_r*d; -d_theta_r*0.5*W_o];

% acceleration of the object on the contact point in object direction
O_acc_co =  - d_R_theta(- theta_r - fai, - d_theta_r - d_fai) * (d_theta_r + d_fai) * d_W_p_o ...
            + R_theta(- theta_r - fai) * dd_W_p_o ...
            + [-dd_theta_r*d; -dd_theta_r*0.5*W_o] ...
            + [-d_theta_r*d_d; 0];

% velocity of the robot on the contact point in world frame
W_v_cr = [d_x_r; d_y_r] + [-d_theta_r*r_r*cos(fai); d_theta_r*r_r*sin(fai)];

% acceleration of the robot on the contact point in world frame
d_W_v_cr = [dd_x_r; dd_y_r] - dd_theta_r*r_r*[cos(fai); -sin(fai)] ...
            + d_fai*d_theta_r*r_r*[sin(fai); cos(fai)];

% velocity of the robot on the contact point in object direction
O_v_cr = R_theta(- theta_r - fai) * W_v_cr;

% acceleration of the robot on the contact point in object direction
O_acc_cr = d_R_theta(- theta_r - fai, - d_theta_r - d_fai) * (- d_theta_r - d_fai) * W_v_cr ...
           + R_theta(- theta_r - fai) * d_W_v_cr;


% second equation
% object velocity at contact point in the x_o direction is the same as the
% one of the robot
% O_v_co = O_v_cr
% eq2 = O_v_co - O_v_cr
% then differentiate this equation so that it is in acceleration domain
eq2 = O_acc_co(1) - O_acc_cr(1);

diff(eq2, dd_x_r)   % 6
diff(eq2, dd_y_r)   % 7
diff(eq2, dd_theta_r)   % 8
diff(eq2, dd_fai)   % 9
diff(eq2, dd_d)     % 10